/*
*
* Author: Giacomo Lozito <james@develia.org>, (C) 2005-2006
*
* This program is free software; you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the
* Free Software Foundation; either version 2 of the License, or (at your
* option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
*
*/


#include "i_vfslayer.h"


#if defined(BUILD_FOR_AUDACIOUS) && !defined(USE_AUDACIOUS12_OR_NEWER)

/* vfs_fstreamopen is a wrapper around vfs_fopen that creates stream api data */
VFSFile * i_vfslayer_fstreamopen( const gchar * filename , const gchar * mode )
{
  VFSFile * vfsfile = vfs_fopen( filename , mode );
  if ( vfsfile != NULL )
  {
    GSList *streamstack = NULL;
    g_dataset_set_data( vfsfile , "s" , streamstack );
    return vfsfile;
  }
  else
    return NULL;
}

/* vfs_fstreamclose is a wrapper around vfs_fclose that frees stream api data */
gint i_vfslayer_fstreamclose( VFSFile * vfsfile )
{
  if ( vfsfile != NULL )
  {
    GSList *streamstack = (GSList*)g_dataset_get_data( vfsfile , "s" );
    if ( streamstack != NULL )
      g_slist_free( streamstack );
    g_dataset_destroy( vfsfile );
  }
  return vfs_fclose( vfsfile );
}


gint i_vfslayer_getc( VFSFile * vfsfile )
{
  /* the code below is definitely not the best for performance,
     but ehy... it's only used by beep media player and by older
     versions of audacious */
  GSList *streamstack = (GSList*)g_dataset_get_data( vfsfile , "s" );
  guchar uc;
  if ( streamstack != NULL ) /* check if some char was ungetc'ed before */
  {
    uc = GPOINTER_TO_INT(streamstack->data);
    streamstack = g_slist_delete_link( streamstack , streamstack );
    g_dataset_set_data( vfsfile , "s" , streamstack );
    return uc;
  }
  else
  {
    if (vfs_fread(&uc, 1, 1, vfsfile))
      return uc;
    else
      return EOF;
  }
}


gint i_vfslayer_ungetc( gint c , VFSFile * vfsfile )
{
  GSList *streamstack = (GSList*)g_dataset_get_data( vfsfile , "s" );
  streamstack = g_slist_prepend( streamstack , GINT_TO_POINTER(c) );
  g_dataset_set_data( vfsfile , "s" , streamstack );
  if ( streamstack != NULL )
    return c;
  else
    return EOF; /* an error occurred */
}


#elif defined(BUILD_FOR_BMP) || defined(BUILD_FOR_XMMS)


VFSFile * i_vfslayer_fopen( const gchar * path , const gchar * mode )
{
  VFSFile *file;
  if (( path == NULL ) || ( mode == NULL ))
    return NULL;
  file = g_new( VFSFile , 1 );
  file->handle = fopen( path , mode );
  if ( file->handle == NULL )
  {
    g_free(file);
    file = NULL;
  }
  return file;
}


gint i_vfslayer_fclose( VFSFile * file )
{
  gint ret = 0;
  if ( file == NULL )
    return -1;
  if (file->handle)
  {
    if ( fclose( file->handle ) != 0 )
      ret = -1;
  }
  g_free(file);
  return ret;
}


size_t i_vfslayer_fread( gpointer ptr , size_t size , size_t nmemb , VFSFile * file )
{
  if ( file == NULL ) return 0;
  return fread( ptr , size , nmemb , file->handle );
}


gint i_vfslayer_fseek( VFSFile * file , glong offset , gint whence )
{
  if ( file == NULL ) return 0;
  return fseek( file->handle , offset , whence );
}


gboolean i_vfslayer_feof( VFSFile * file )
{
  if ( file == NULL ) return FALSE;
  return (gboolean)feof( file->handle );
}


gint i_vfslayer_getc( VFSFile * file )
{
  return getc( file->handle );
}


gint i_vfslayer_ungetc( gint c , VFSFile * file )
{
  return ungetc( c , file->handle );
}

#endif /* defined(BUILD_FOR_BMP) || defined(BUILD_FOR_XMMS) */
